# Note: plugins are loaded from 'internal-tests.sh'

# Nix
{ lib
, stdenvNoCC # for E-ACSL
, fetchurl
, gitignoreSource
, makeWrapper
, nix-gitignore
, wrapGAppsHook
, writeText
# Generic
, findlib
# Frama-C build
, apron
, camlzip
, camomile
, dune_3
, dune-configurator
, dune-site
, gcc9
, graphviz
, lablgtk3
, lablgtk3-sourceview3
, ltl2ba
, menhir
, menhirLib
, mlmpfr
, ocaml
, ocamlgraph
, ocp-indent
, ppx_deriving
, ppx_deriving_yojson
, ppx_import
, yojson
, which
, why3
, zarith
, zmq
# Frama-C tests
, alt-ergo
, dos2unix
, doxygen
, perl
, pkgs
, python3
, swiProlog
, time
}:

# We do not use buildDunePackage because Frama-C still uses a Makefile to build
# some files and prepare some information before starting dune.
stdenvNoCC.mkDerivation rec {
  pname = "frama-c-internal-tests";
  version =
    lib.strings.replaceStrings ["~"] ["-"]
      (lib.strings.removeSuffix "\n"
        (builtins.readFile ../VERSION));
  slang = lib.strings.removeSuffix "\n" (builtins.readFile ../VERSION_CODENAME);

  src = gitignoreSource ./..;

  nativeBuildInputs = [
    which
    wrapGAppsHook
  ];

  buildInputs = [
    apron
    alt-ergo
    camlzip
    camomile
    dune_3
    dune-configurator
    dune-site
    findlib
    gcc9
    graphviz
    lablgtk3
    lablgtk3-sourceview3
    ltl2ba
    menhir
    menhirLib
    mlmpfr
    ocaml
    ocamlgraph
    ocp-indent
    ppx_deriving
    ppx_deriving_yojson
    ppx_import
    yojson
    which
    why3
    zarith
    zmq
    # Tests
    alt-ergo
    dos2unix
    doxygen
    perl
    pkgs.getopt
    python3
    swiProlog
    time
  ];

  outputs = [ "out" ];

  preConfigure = ''
    patchShebangs src/plugins/eva/gen-api.sh
    chmod +x src/plugins/eva/gen-api.sh
    dune build @frama-c-configure
  '';

  # Do not use default parallel building, but allow 2 cores for Frama-C build
  enableParallelBuilding = false;
  buildPhase = ''
    dune build -j2 --display short --error-reporting=twice @install
    make tools/ptests/ptests.exe
    make tools/ptests/wtests.exe
  '';

  wp_cache = fetchGit {
           url = "git@git.frama-c.com:frama-c/wp-cache.git" ;
           ref = "master" ;
           shallow = true ;
         };

  doCheck = true;
  preCheck = ''
    patchShebangs .
    mkdir home
    HOME=$(pwd)/home
    why3 config detect
    export FRAMAC_WP_CACHE=offline
    export FRAMAC_WP_CACHEDIR=$wp_cache
  '';

  checkPhase = ''
    runHook preCheck
    dune exec -- frama-c-ptests -never-disabled tests src/plugins/*/tests
    dune build -j1 --display short @ptests_config
  '';

  installFlags = [
    "PREFIX=$(out)"
  ];

  meta = {
    description = "An extensible and collaborative platform dedicated to source-code analysis of C software";
    homepage = "http://frama-c.com/";
    license = lib.licenses.lgpl21;
    platforms = lib.platforms.unix;
  };
}
